@@ -1,18 +1,23 @@
-(************************************************************************)
-(* This file is part of SKS.  SKS is free software; you can
-   redistribute it and/or modify it under the terms of the GNU General
-   Public License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA *)
+(***********************************************************************)
+(* pMap.ml - Association tables over ordered types.                    *)
+(*                                                                     *)
+(* Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, *)
+(*               2011, 2012  Yaron Minsky and Contributors             *)
+(*                                                                     *)
+(* This file is part of SKS.  SKS is free software; you can            *)
+(* redistribute it and/or modify it under the terms of the GNU General *)
+(* Public License as published by the Free Software Foundation; either *)
+(* version 2 of the License, or (at your option) any later version.    *)
+(*                                                                     *)
+(* This program is distributed in the hope that it will be useful, but *)
+(* WITHOUT ANY WARRANTY; without even the implied warranty of          *)
+(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   *)
+(* General Public License for more details.                            *)
+(*                                                                     *)
+(* You should have received a copy of the GNU General Public License   *)
+(* along with this program; if not, write to the Free Software         *)
+(* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 *)
+(* USA or see <http://www.gnu.org/licenses/>.                          *)
 (***********************************************************************)
 
 open StdLabels
@@ -37,10 +42,10 @@ module type S =
     val map: f:('data -> 'a) -> ('key,'data) t -> ('key,'a) t
     val mapi: f:(key:'key -> data:'data -> 'a) ->
       ('key,'data) t -> ('key,'a) t
-    val fold: f:(key:'key -> data:'data -> 'a -> 'a) -> 
+    val fold: f:(key:'key -> data:'data -> 'a -> 'a) ->
       ('key,'data) t -> init:'a -> 'a
     val of_alist: ('key * 'data) list -> ('key,'data) t
-    val to_alist: ('key,'data) t -> ('key * 'data) list 
+    val to_alist: ('key,'data) t -> ('key * 'data) list
   end
 
 module Make(Ord: OrderedType) = struct
@@ -146,8 +151,8 @@ module Make(Ord: OrderedType) = struct
 
     let rec mapi ~f = function
         Empty               -> Empty
-      | Node(l, v, d, r, h) -> 
-	  Node(mapi ~f l, v, f ~key:v ~data:d, mapi ~f r, h)
+      | Node(l, v, d, r, h) ->
+          Node(mapi ~f l, v, f ~key:v ~data:d, mapi ~f r, h)
 
     let rec fold ~f m ~init:accu =
       match m with
@@ -155,13 +160,13 @@ module Make(Ord: OrderedType) = struct
       | Node(l, v, d, r, _) ->
           fold ~f l ~init:(f ~key:v ~data:d (fold ~f r ~init:accu))
 
-    let of_alist alist = 
+    let of_alist alist =
       List.fold_left ~f:(fun map (key,data) -> add ~key ~data map)
-	~init:empty alist
+        ~init:empty alist
 
-    let to_alist map = 
+    let to_alist map =
       fold ~f:(fun ~key ~data list -> (key,data)::list)
-	~init:[] map
+        ~init:[] map
 end
 
 module Map = Make(ClassicalType)
